/// @file       wifidemo_wince7.c
/// @copyright  Copyright (c) 2014 Toradex AG
/// @author     $ guc $
/// @date       $ 8/5/2014 $
/// @brief      This is a demo code to demonstrate how to connect to a preferred wireless access point
///             (Wi-Fi) from program on WinCE 7.

#include <windows.h> 
/// Header file for Native Wi-Fi
#include "Wlanapi.h"

#pragma comment(lib, "wlanapi.lib")
/// Declarations
GUID iGUID;
PHANDLE phClientHandle;
PDWORD dwNegotiatedVersion;
PWLAN_BSS_LIST ppWlanBssList;
PWLAN_INTERFACE_INFO pSelectedInterface;
PWLAN_INTERFACE_INFO_LIST ppInterfaceList;
WLAN_CONNECTION_PARAMETERS pConnectionParameter;
PWLAN_AVAILABLE_NETWORK_LIST ppAvailableNetworkList;

int wmain(void)
{
    char xmlProfile[1024];
    char password[30];
    char p_name[30] = "User_Profile";
    char securityType[30];
    UCHAR ssidName[50];
    WCHAR buf[2048];
    WCHAR wpname[128];
    int userMenuInput = 0;
    int userPreferredNetwork = 0;
    int securityTypeNumber = 0;
    DWORD reasonCode;
    DWORD tempCount1 = 0;
    DWORD tempCount2 = 0; 
    DWORD dwReturn;

    printf("Wi-Fi Connection from user application\n");

    /// Create a handle to the Client
    dwReturn = WlanOpenHandle(WLAN_API_VERSION_1_0, NULL, &dwNegotiatedVersion, &phClientHandle);

    printf("\nConnecting to Native Wi-Fi.........");
    if (dwReturn == ERROR_SUCCESS) 
    {
        printf("Handle created!");
    }
    else
    {
        printf("Error code %d, could not create Handle!", dwReturn);
        getchar();
        return(FALSE);
    }

    /// Listing the Wi-Fi interface 
    printf("\nListing interfaces.................");

    dwReturn = WlanEnumInterfaces(phClientHandle, NULL, &ppInterfaceList);

    if (dwReturn == ERROR_SUCCESS) 
    {
        printf("Interface List Retrieved!");
    }
    else
    {
        printf("Error code %d could not capture Interface List!", dwReturn);
        getchar();
        return(FALSE);
    }

    for (tempCount1 = 0; tempCount1 < ppInterfaceList->dwNumberOfItems; tempCount1++)
    {
        printf("\nInterface %d:", tempCount1);
        for (tempCount2 = 0; tempCount2 < 10; tempCount2++)
        {
            printf("%c", ppInterfaceList->InterfaceInfo[tempCount1].strInterfaceDescription[tempCount2]);
        }
    }

    pSelectedInterface = (WLAN_INTERFACE_INFO*) &ppInterfaceList->InterfaceInfo[0];

    /// Listing the visible networks in interface index 0
    /// Listing the Wi-Fi interface 
    printf("\nListing wireless networks..........");
    dwReturn = WlanGetAvailableNetworkList(phClientHandle, 
                                           &pSelectedInterface->InterfaceGuid,
                                           0,
                                           NULL,
                                           &ppAvailableNetworkList);

    if (dwReturn == ERROR_SUCCESS) 
    {
        printf("SSID of wireless network obtained!\n");
    }
    else
    {
        printf("Error code %d could not capture Interface List!\n", dwReturn);
        getchar();
        return(FALSE);
    }
    while (1)
    {
        /// Display available wireless networks
        for (tempCount1 = 0; tempCount1 < ppAvailableNetworkList->dwNumberOfItems; tempCount1++)
        {
            printf("\nSSID %d: ", tempCount1);
            printf("%s", ppAvailableNetworkList->Network[tempCount1].dot11Ssid.ucSSID);
            printf("  Signal strength: %d", ppAvailableNetworkList->Network[tempCount1].wlanSignalQuality); 
        }
        printf("\nEnter your choose:\n1) Connect to preferred network\n2) Disconnect from network\n3) Exit\n");
        scanf_s("%d", &userMenuInput);

        switch (userMenuInput)
        {
            case 1:      ///< Connect to preferred network
                printf("\nEnter SSID number for the preferred network: ");
                scanf_s(" %d", &userPreferredNetwork);
                if (userPreferredNetwork > ppAvailableNetworkList->dwNumberOfItems)
                {
                    printf("\nEnter valid choose");
                    break;
                }
                strcpy_s(ssidName, 50, ppAvailableNetworkList->Network[userPreferredNetwork].dot11Ssid.ucSSID);
                
                printf("\nEnter the password: ");
                scanf_s(" %[^\n]s", password);
                printf("\nEnter security type:\n 1) WPAPSK 2) WPA2PSK\t");
                scanf_s(" %d", &securityTypeNumber);
                if (securityTypeNumber == 1)
                {
                    strcpy_s(securityType, 30, "WPAPSK");
                }
                else if (securityTypeNumber == 2)
                {
                    strcpy_s(securityType, 30, "WPA2PSK");
                }
                else 
                {
                    printf("\nEnter valid choose");
                    break;
                }

                /// Set the XML profile for the preferred network 
                sprintf_s(xmlProfile, 1024, "<?xml version=\"1.0\"?><WLANProfile xmlns=\"http://www.microsoft.com/networking/WLAN/profile/v1\"><name>%s</name><SSIDConfig><SSID><name>%s</name></SSID><nonBroadcast>false</nonBroadcast></SSIDConfig><connectionType>ESS</connectionType><connectionMode>auto</connectionMode><autoSwitch>false</autoSwitch><MSM><security><authEncryption><authentication>%s</authentication><encryption>AES</encryption><useOneX>false</useOneX><FIPSMode xmlns=\"http://www.microsoft.com/networking/WLAN/profile/v2\">false</FIPSMode></authEncryption><sharedKey><keyType>passPhrase</keyType><protected>false</protected><keyMaterial>%s</keyMaterial></sharedKey></security></MSM></WLANProfile>",
                p_name, ppAvailableNetworkList->Network[userPreferredNetwork].dot11Ssid.ucSSID, securityType, password);

                memset(buf, 0, sizeof(buf));
                MultiByteToWideChar(CP_ACP, 0, xmlProfile, strlen(xmlProfile)+1, buf, sizeof(buf)/sizeof(buf[0]));
                /// Set network profile
                dwReturn = WlanSetProfile(phClientHandle, &pSelectedInterface->InterfaceGuid, 0x00, buf, NULL, TRUE, NULL, &reasonCode);
                if (reasonCode == WLAN_REASON_CODE_SUCCESS)
                {
                    printf("\nProfile set\n");
                }
                MultiByteToWideChar(CP_ACP, 0, p_name, strlen(p_name)+1, wpname, sizeof(wpname)/sizeof(wpname[0]));

                /// Get BSS list of the network
                dwReturn = WlanGetNetworkBssList(phClientHandle, 
                                                 &pSelectedInterface->InterfaceGuid, 
                                                 &ppAvailableNetworkList->Network[0].dot11Ssid,
                                                 ppAvailableNetworkList->Network[0].dot11BssType, 
                                                 1, 
                                                 NULL, 
                                                 &ppWlanBssList);
                if (dwReturn != ERROR_SUCCESS)
                {
                    printf("Could not obtain BSS list\n");
                    break;
                }

                /// Set connection parameter
                pConnectionParameter.dot11BssType = ppAvailableNetworkList->Network[1].dot11BssType;
                pConnectionParameter.dwFlags = WLAN_CONNECTION_HIDDEN_NETWORK;
                pConnectionParameter.pDesiredBssidList = NULL; 
                pConnectionParameter.pDot11Ssid = NULL;
                pConnectionParameter.strProfile = wpname;
                pConnectionParameter.wlanConnectionMode = wlan_connection_mode_profile;

                /// Connect to network
                dwReturn = WlanConnect(phClientHandle, 
                                         &pSelectedInterface->InterfaceGuid,
                                         &pConnectionParameter,
                                         NULL);
                if (dwReturn == ERROR_SUCCESS)
                {
                    printf("Connected successfully\n");
                }
                else
                {
                    printf("\nError connecting network");
                }
                break;
            case 2:     ///< Disconnect from preferred network
                dwReturn = WlanDisconnect(phClientHandle, &pSelectedInterface->InterfaceGuid, NULL); 
                if (dwReturn == ERROR_SUCCESS)
                {
                    printf("\nDisconnected successfully");
                }
                break;
            case 3:    ///< Exit
                exit(0);
                break;
            default:
                printf("Please enter valid choose\n");
                break;
        }
    }
    printf("\nPress Enter to end the program\n\r");
    getchar();

    return(TRUE);
}
